/*
 * options.c - Copyright (C) 1998 Pat Thoyts <pat@calubc.caladan.co.uk>
 *
 * Process options for sbmixer. The usage statement is also in this file.
 *
 * $Id: options.c,v 0.2 1997/02/04 18:23:32 pat Exp $
 *
 * pat: Hmm This was obviously written before I discovered the joys of
 *      getopt.h however, as it works fine I don't think I'll mess with it.
 *
 */

#include <string.h>
#include <ctype.h>
#include "sbmixer.h"

extern struct flag_struct flag;

static int do_short_options(int counter, int argc, char **argv,
			    struct mixer_struct *device);
static int do_x_options(int counter, int argc, char **argv,
			struct mixer_struct *device);
static int eval_option(const char *opt, int counter, int argc, char **argv,
		       char operr);

void
get_options(int argc, char **argv, struct mixer_struct *device)
{
  int counter;

  if ((argc > 1) && ((argv[1][0]=='?') || (argv[1][1]=='?')))
    usage(argv[0]);
  for (counter = 1; counter != argc; counter++) {
    if (argv[counter][0]=='/')
      do_x_options(counter, argc, argv, device);
    else if (argv[counter][0]=='-') {
      if(argv[counter][1]=='-')
	do_x_options(counter, argc, argv, device);
      else
	do_short_options(counter, argc, argv, device);
    }
  }
}

static int
do_short_options(int counter, int argc, char **argv,
		 struct mixer_struct *device)
{
  extern const unsigned char *internal_name, *version_string, *gnutext;
  int skip, op_value;
  const char *filename, *opt;
  char letter;

  opt = argv[counter];
  skip = 0;
  for (++opt;((skip == 0) && (*opt != '\0')); opt++) {
    switch(*opt) {
    case 'f':
      ++opt;
      if (*opt == '\0') {
	if ((counter +1) < argc)
	  filename = argv[++counter];
	else {
	  fprintf(stderr, "option `-f' requires an argument");
	  fflush(stderr);
	  usage(argv[0]);
	}
      }
      else
	filename = opt;
      skip = 1;
      break;
    case 'm': case 'b': case 't': case 'l': case 'c': case 'v':
    case 'i': case 'k': case 'r': case 'C': case 'x': case '2': 
      letter = *opt;
      op_value = eval_option(opt, counter, argc, argv, letter);
      flag.change[flag.nr_changes].id = letter;
      flag.change[flag.nr_changes].new = op_value;
      ++flag.nr_changes;
      if(flag.nr_changes > SOUND_MIXER_NRDEVICES) {
	fprintf(stderr, "Oops. More changes than devices! Array is too small"
		" to hold all the changes.\nTo fix, use a smaller command"
		"line. Do you really need all that?!\n");
	exit(3);
      }
      skip = 1;
      break;
    case 'D':
      ++flag.debug;
      break;
    case 'S':
      flag.verbose = 1;
      break;
    case 'L':
      flag.large_range = 1;
      break;
    case 'V':
      printf("%s version %s "CCUSED"\n", internal_name, version_string);
      break;
    case 'h': case 'H': case 'U': case 'u': case '?':
      usage(argv[0]);
      break;
    case 'G':
      printf("%s",gnutext);
      exit(0);
      break;
    default:
      fprintf (stderr, "Unknown option '-%c'.\n", *opt);
      break;
    }
  }
  return 1;
}

static int
do_x_options(int counter, int argc, char **argv, struct mixer_struct *device)
{
  extern const unsigned char *internal_name, *version_string, *gnutext;
  const char *opt, *p_opt = '\0';
  #define NAMES 20
  const char ids[NAMES] = {'m','m','m','b','t','v','i',
		        'k','l','C','c','x','2','r',
		        'G','U','D','V','S','L'};
  const char *names[NAMES] = {"vol","master","m", "bass", "treble", "voc",
				"midi", "speaker", "line", "mic", "cd",
				"rmix", "midi2", "rec", "gnu", "help",
				"debug", "version", "verbose","large"};
  char op_id = 0;
  int op_value, op_length, cn;

  opt = argv[counter];
  if(*opt=='/') ++opt;
  else opt += 2;
  if((p_opt = strchr(opt, ':')) != NULL)
    op_length = p_opt - opt;
  else {
    p_opt = "";
    op_length = strlen(opt);
  }
  for(cn = 0; cn < NAMES; cn++) {
    if (strncasecmp(names[cn], opt, op_length) == 0) op_id = ids[cn];
  }
  switch(op_id) {
  case 'G':
    printf("%s",gnutext);
    exit(0);
    break;
  case 'U':
    usage(argv[0]);
    break;
  case 'D':
    ++flag.debug;
    break;
  case 'V':
    printf("%s version %s "CCUSED"\n", internal_name, version_string);
    break;
  case 'S':
    flag.verbose = 1;
    break;
  case 'L':
    flag.large_range = 1;
    break;
  case 'm': case 'b': case 't': case 'v': case 'i': case 'k':
  case 'l': case 'C': case 'c': case 'x': case '2': case 'r':
    if (*p_opt == ':') opt = p_opt;
    else opt = ".";
    op_value = eval_option(opt, counter, argc, argv, op_id);
    flag.change[flag.nr_changes].id = op_id;
    flag.change[flag.nr_changes].new = op_value;
    ++flag.nr_changes;
    if(flag.nr_changes > SOUND_MIXER_NRDEVICES) {
      fprintf(stderr, "Oops. More changes than devices! Array is too small"
	      " to hold all the changes.\nTo fix, use a smaller command"
	      "line. Do you really need all that?!\n");
      exit(3);
    }
    break;
  default:
    fprintf (stderr, "Unknown option `--%s'.\n", opt);
    break;
  }
  return 1;
}

static int
eval_option(const char *opt, int counter, int argc, char **argv, char operr)
{
  int value, left, right;
  const char *buffer = '\0', *p_buffer;

  ++opt;
  if (*opt == '\0') {
    if ((counter +1) < argc)
      buffer = argv[++counter];
    else {
      fprintf(stderr, "option `-%c' requires an argument.\n", operr);
      fflush(stderr);
      usage(argv[0]);
    }
  }
  else
    buffer = opt;
  if (*buffer == ':') ++buffer;
  if (strchr(buffer, ',') == NULL) {
    value = strtol(buffer, NULL, 0);
    value = (value*256)+value;
  }
  else {
    left = strtol(buffer, NULL , 0);
    p_buffer = strrchr(buffer, ',');
    right = strtol(++p_buffer, NULL, 0);
    value = (left*256)+right;
  }
  if (value > 0x6464) flag.large_range = 1;
  return value;
}


void
usage(char *program_name)
{
  extern const unsigned char *version_string, *internal_name;
  char *buffer;

  if ((buffer = strrchr(program_name, '/')))
    program_name = buffer + 1;
  else if ((buffer = strrchr(program_name, '\\')))
    program_name = buffer + 1;
  fprintf(stderr, "%s version %s "CCUSED, internal_name, version_string);
  fprintf(stderr,
	  "\nusage: %s [-?huGVDS] -option LEVEL -option LEFT,RIGHT] "
	  "\n"
	  "\n       -h -? --help     show this usage information."
	  "\n       -D    --debug    show debugging info (very verbose)"
	  "\n       -G    --gnu      display GNU public license information"
	  "\n       -V    --version  display version and compilation information."
	  "\n       -S    --verbose  be verbose (default is quiet)"
	  "\n       -L    --large    use 0-255 value range (for SB16 DOS compatability.)"
	  "\n\n       -m    --vol      change the master volume"
	  "\n       -b    --bass              bass level"
	  "\n       -t    --treble            treble"
	  "\n       -c    --cd                CD"
	  "\n       -v    --voc               voc"
	  "\n       -i    --midi              midi"
	  "\n       -l    --line              line"
	  "\n       -x    --rmix              recording mixer"
	  "\n       -2    --midi2             second midi device"
	  "\n       -C    --mic               mic"
	  "\n       -r    --rec               recording level"
	  "\n\n   Note that not all devices will be supported for all cards."
	  "\n", program_name);
  fflush(stderr);
  exit(1);
}
